"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.onPostBootstrap = void 0;
const crypto_1 = __importDefault(require("crypto"));
const natural_1 = require("natural");
const constants_1 = require("./constants");
const compute_cosine_similarity_1 = __importDefault(require("compute-cosine-similarity"));
const vector_similarity_memo = new Map();
const md5 = (str) => {
    const md5 = crypto_1.default.createHash("md5");
    return md5.update(str, "binary").digest("hex");
};
const getMemorizedVectorSimilarity = (v1, v2) => {
    const id = v1.id < v2.id ? `${v1.id} ${v2.id}` : `${v2.id} ${v1.id}`;
    const memorized_similarity = vector_similarity_memo.get(id);
    if (memorized_similarity !== undefined)
        return memorized_similarity;
    const similarity = calcVectorSimilarity(v1.vector, v2.vector);
    vector_similarity_memo.set(id, similarity);
    return similarity;
};
const calcVectorSimilarity = (v1, v2) => {
    if (v1.length !== v2.length)
        throw new Error("Both vector's size must be equal");
    return (0, compute_cosine_similarity_1.default)(v1, v2) || 0;
};
const getRelatedPosts = (id, bow_vectors) => {
    const vector = bow_vectors.get(id);
    if (vector === undefined)
        return [];
    const vector_node = {
        id,
        vector,
    };
    return Array.from(bow_vectors.entries())
        .sort((x, y) => {
        const vector_x = {
            id: x[0],
            vector: x[1],
        };
        const vector_y = {
            id: y[0],
            vector: y[1],
        };
        return (getMemorizedVectorSimilarity(vector_y, vector_node) -
            getMemorizedVectorSimilarity(vector_x, vector_node));
    })
        .map((x) => x[0]);
};
const getTextFromRawText = async (doc) => {
    return doc
        .replace(/http[^ ]+/g, "")
        .replace(/[\#\!\(\)\*\_\[\]\|\=\>\+\`\:\-]/g, "");
};
const getSpaceSeparatedDoc = async (doc) => {
    return "";
};
const onPostBootstrap = async ({ actions, getNode, getNodesByType, createNodeId, reporter, cache }, options) => {
    const { etriToken } = options;
    if (etriToken === "") {
        reporter.info("[Related Post] Skip getting related post logic.");
        return;
    }
    const nodes = getNodesByType(constants_1.NODE_TYPE.Post);
    const docs = nodes.map((node) => ({
        id: node.id,
        text: node.rawText,
    }));
    const tfidf = new natural_1.TfIdf();
    // tfidf
    docs.map(async (doc) => {
        if (doc.text) {
            const key = `${md5(doc.text)}-doc`;
            const cached_ssd = await cache.get(key);
            if (cached_ssd !== undefined) {
                tfidf.addDocument(cached_ssd);
            }
            else {
                const ssd = await getSpaceSeparatedDoc(await getTextFromRawText(doc.text));
                tfidf.addDocument(ssd);
                await cache.set(key, ssd);
            }
        }
    });
    //
    const doc_terms = docs.map((_, i) => tfidf.listTerms(i)
        .map((x) => ({ ...x, tfidf: x.tf * x.idf }))
        .sort((x, y) => y.tfidf - x.tfidf));
    const all_keywords = new Set();
    const tfidf_map_for_each_doc = [];
    doc_terms.forEach((x, i) => {
        tfidf_map_for_each_doc[i] = new Map();
        x.slice(0, 30).forEach((x) => {
            all_keywords.add(x.term);
            tfidf_map_for_each_doc[i].set(x.term, x.tfidf);
        });
    });
    const bow_vectors = new Map();
    docs.forEach((x, i) => {
        if (bow_vectors === null)
            return;
        bow_vectors.set(x.id, Array.from(all_keywords)
            .map((x) => tfidf_map_for_each_doc[i].get(x))
            .map((x) => (x === undefined ? 0 : x)));
    });
    reporter.info(`[related-posts] bow vectors generated, dimention: ${all_keywords.size}`);
    nodes.forEach((node) => {
        const related_nodes = getRelatedPosts(node.id, bow_vectors)
            .slice(1)
            .map((id) => getNode(id));
        const digest = `${node.id} - ${constants_1.NODE_TYPE.RelatedPost}`;
        actions.createNode({
            id: createNodeId(digest),
            parent: node.id,
            internal: {
                type: `related${constants_1.NODE_TYPE.RelatedPost}s`,
                contentDigest: digest,
            },
            posts: related_nodes,
        });
    });
};
exports.onPostBootstrap = onPostBootstrap;
